@@ -1,30 +1,23 @@
 #
-# state.rb
+# $Id: state.rb,v 1.3 2005/11/20 17:31:32 aamine Exp $
 #
-#   Copyright (c) 1999-2001 Minero Aoki <aamine@loveruby.net>
+# Copyright (c) 1999-2005 Minero Aoki
 #
-#   This program is free software.
-#   You can distribute/modify this program under the terms of
-#   the GNU Lesser General Public License version 2 or later.
+# This program is free software.
+# You can distribute/modify this program under the terms of
+# the GNU LGPL, Lesser General Public License version 2.1.
+# For details of the GNU LGPL, see the file "COPYING".
 #
 
 require 'racc/iset'
-
+require 'racc/exception'
 
 module Racc
 
-  class RaccError < StandardError; end
-
-
-  #
-  # StateTable
-  #
-  # the table of lalr states.
-  #
-
+  # A table of LALR states.
   class StateTable
 
-    def initialize( racc )
+    def initialize(racc)
       @ruletable   = racc.ruletable
       @symboltable = racc.symboltable
 
@@ -35,10 +28,10 @@ module Racc
       @states = []
       @statecache = {}
 
-      @actions = ActionTable.new( @ruletable, self )
+      @actions = ActionTable.new(@ruletable, self)
     end
 
-    attr :actions
+    attr_reader :actions
 
     def size
       @states.size
@@ -50,32 +43,31 @@ module Racc
 
     alias to_s inspect
 
-    def []( i )
+    def [](i)
       @states[i]
     end
 
-    def each_state( &block )
-      @states.each( &block )
+    def each_state(&block)
+      @states.each(&block)
     end
 
     alias each each_state
 
-    def each_index( &block )
-      @states.each_index( &block )
+    def each_index(&block)
+      @states.each_index(&block)
     end
 
-
-    ###
-    ### nfa
-    ###
+    #
+    # nfa
+    #
 
     def init
       # add state 0
-      core_to_state( [ @ruletable[0].ptrs[0] ] )
+      core_to_state  [ @ruletable[0].ptrs[0] ]
 
       cur = 0
       @gotos = []
-      while cur < @states.size do
+      while cur < @states.size
         generate_states @states[cur]   # state is added here
         cur += 1
       end
@@ -83,14 +75,14 @@ module Racc
       @actions.init
     end
 
-    def generate_states( state )
+    def generate_states(state)
       puts "dstate: #{state}" if @d_state
 
       table = {}
       ptr = pt = s = g = nil
 
       state.closure.each do |ptr|
-        if sym = ptr.deref then
+        if sym = ptr.dereference
           addsym table, sym, ptr.next
         end
       end
@@ -98,48 +90,47 @@ module Racc
       table.each do |sym, core|
         puts "dstate: sym=#{sym} ncore=#{core}" if @d_state
 
-        dest = core_to_state( core.to_a )
+        dest = core_to_state(core.to_a)
         state.goto_table[sym] = dest
-        id = if sym.nonterminal? then @gotos.size else nil end
-        g = Goto.new( id, sym, state, dest )
-        if sym.nonterminal? then
-          @gotos.push g
-        end
+        id = sym.nonterminal?() ? @gotos.size : nil
+        g = Goto.new(id, sym, state, dest)
+        @gotos.push g if sym.nonterminal?
         state.gotos[sym] = g
         puts "dstate: #{state.ident} --#{sym}--> #{dest.ident}" if @d_state
 
         # check infinite recursion
-        if state.ident == dest.ident and state.closure.size == 1 then
-          raise RaccError,
-            sprintf( "Infinite recursion: state %d, with rule %d",
-                     state.ident, state.ptrs[0].rule.ident )
+        if state.ident == dest.ident and state.closure.size == 1
+          raise CompileError,
+              sprintf("Infinite recursion: state %d, with rule %d",
+                      state.ident, state.ptrs[0].rule.ident)
         end
       end
     end
 
-    def addsym( table, sym, ptr )
-      unless s = table[sym] then
+    def addsym(table, sym, ptr)
+      unless s = table[sym]
         table[sym] = s = ISet.new
       end
       s.add ptr
     end
 
-    def core_to_state( core )
+    def core_to_state(core)
       #
-      # creates/find state keeping core unique
+      # convert CORE to a State object.
+      # If matching state does not exist, create it and add to the table.
       #
 
-      k = fingerprint( core )
-      unless dest = @statecache[k] then
+      k = fingerprint(core)
+      unless dest = @statecache[k]
         # not registered yet
-        dest = State.new( @states.size, core )
+        dest = State.new(@states.size, core)
         @states.push dest
 
         @statecache[k] = dest
         
         puts "core_to_state: create state   ID #{dest.ident}" if @d_state
       else
-        if @d_state then
+        if @d_state
           puts "core_to_state: dest is cached ID #{dest.ident}"
           puts "core_to_state: dest core #{dest.core.join(' ')}"
         end
@@ -148,14 +139,13 @@ module Racc
       dest
     end
 
-    def fingerprint( arr )
-      arr.collect {|i| i.ident }.pack( 'L*' )
+    def fingerprint(arr)
+      arr.map {|i| i.ident }.pack('L*')
     end
 
-
-    ###
-    ### dfa
-    ###
+    #
+    # dfa
+    #
 
     def determine
       la = lookahead
@@ -171,11 +161,6 @@ module Racc
       check_useless
     end
 
-
-    #
-    # lookahead
-    #
-
     def lookahead
       #
       # lookahead algorithm ver.3 -- from bison 1.26
@@ -183,7 +168,7 @@ module Racc
       state = goto = arr = ptr = st = rl = i = a = t = g = nil
 
       gotos = @gotos
-      if @d_la then
+      if @d_la
         puts "\n--- goto ---"
         gotos.each_with_index {|g,i| print i, ' '; p g }
       end
@@ -192,23 +177,22 @@ module Racc
       ### set_goto_map()
       la_rules = []
       @states.each do |state|
-        state.check_la( la_rules )
+        state.check_la la_rules
       end
 
-
       ### initialize_F()
-      f     = create_tmap( gotos.size )
+      f     = create_tmap(gotos.size)
       reads = []
       edge  = []
       gotos.each do |goto|
         goto.to_state.goto_table.each do |t, st|
-          if t.terminal? then
+          if t.terminal?
             f[goto.ident] |= (1 << t.ident)
-          elsif t.nullable? then
+          elsif t.nullable?
             edge.push goto.to_state.gotos[t].ident
           end
         end
-        if edge.empty? then
+        if edge.empty?
           reads.push nil
         else
           reads.push edge
@@ -216,24 +200,23 @@ module Racc
         end
       end
       digraph f, reads
-      if @d_la then
+      if @d_la
         puts "\n--- F1 (reads) ---"
         print_tab gotos, reads, f
       end
 
-
       ### build_relations()
       ### compute_FOLLOWS
       path = nil
       edge = []
-      lookback = Array.new( la_rules.size, nil )
+      lookback = Array.new(la_rules.size, nil)
       includes = []
       gotos.each do |goto|
         goto.symbol.heads.each do |ptr|
-          path = record_path( goto.from_state, ptr.rule )
+          path = record_path(goto.from_state, ptr.rule)
           g = path[-1]
           st = g ? g.to_state : goto.from_state
-          if st.conflict? then
+          if st.conflict?
             addrel lookback, st.rruleid(ptr.rule), goto
           end
           path.reverse_each do |g|
@@ -242,31 +225,30 @@ module Racc
             break unless g.symbol.nullable?
           end
         end
-        if edge.empty? then
+        if edge.empty?
           includes.push nil
         else
           includes.push edge
           edge = []
         end
       end
-      includes = transpose( includes )
+      includes = transpose(includes)
       digraph f, includes
-      if @d_la then
+      if @d_la
         puts "\n--- F2 (includes) ---"
         print_tab gotos, includes, f
       end
 
-
       ### compute_lookaheads
-      la = create_tmap( la_rules.size )
+      la = create_tmap(la_rules.size)
       lookback.each_with_index do |arr, i|
-        if arr then
+        if arr
           arr.each do |g|
             la[i] |= f[g.ident]
           end
         end
       end
-      if @d_la then
+      if @d_la
         puts "\n--- LA (lookback) ---"
         print_tab la_rules, lookback, la
       end
@@ -274,19 +256,19 @@ module Racc
       la
     end
     
-    def create_tmap( siz )
-      Array.new( siz, 0 )   # use Integer as bitmap
+    def create_tmap(size)
+      Array.new(size, 0)   # use Integer as bitmap
     end
 
-    def addrel( tbl, i, item )
-      if a = tbl[i] then
+    def addrel(tbl, i, item)
+      if a = tbl[i]
         a.push item
       else
         tbl[i] = [item]
       end
     end
 
-    def record_path( begst, rule )
+    def record_path(begst, rule)
       st = begst
       path = []
       rule.symbols.each do |t|
@@ -297,10 +279,10 @@ module Racc
       path
     end
 
-    def transpose( rel )
-      new = Array.new( rel.size, nil )
+    def transpose(rel)
+      new = Array.new(rel.size, nil)
       rel.each_with_index do |arr, idx|
-        if arr then
+        if arr
           arr.each do |i|
             addrel new, i, idx
           end
@@ -309,31 +291,31 @@ module Racc
       new
     end
 
-    def digraph( map, relation )
+    def digraph(map, relation)
       n = relation.size
-      index    = Array.new( n, nil )
+      index    = Array.new(n, nil)
       vertices = []
       @infinity = n + 2
 
       i = nil
       index.each_index do |i|
-        if not index[i] and relation[i] then
+        if not index[i] and relation[i]
           traverse i, index, vertices, map, relation
         end
       end
     end
 
-    def traverse( i, index, vertices, map, relation )
+    def traverse(i, index, vertices, map, relation)
       vertices.push i
       index[i] = height = vertices.size
 
       proci = nil
-      if rp = relation[i] then
+      if rp = relation[i]
         rp.each do |proci|
-          unless index[proci] then
+          unless index[proci]
             traverse proci, index, vertices, map, relation
           end
-          if index[i] > index[proci] then
+          if index[i] > index[proci]
             # circulative recursion !!!
             index[i] = index[proci]
           end
@@ -341,8 +323,8 @@ module Racc
         end
       end
 
-      if index[i] == height then
-        while true do
+      if index[i] == height
+        while true
           proci = vertices.pop
           index[proci] = @infinity
           break if i == proci
@@ -352,78 +334,79 @@ module Racc
       end
     end
 
-    ###
-
-    def print_atab( idx, tab )
+    # for debug
+    def print_atab(idx, tab)
       tab.each_with_index do |i,ii|
         printf '%-20s', idx[ii].inspect
         p i
       end
     end
 
-    def print_tab( idx, rel, tab )
+    def print_tab(idx, rel, tab)
       tab.each_with_index do |bin,i|
         print i, ' ', idx[i].inspect, ' << '; p rel[i]
         print '  '
-        each_t( @symboltable, bin ) {|t| print ' ', t }
+        each_t(@symboltable, bin) {|t| print ' ', t }
         puts
       end
     end
-def print_tab_i( idx, rel, tab, i )
-  bin = tab[i]
-  print i, ' ', idx[i].inspect, ' << '; p rel[i]
-  print '  '
-  each_t( @symboltable, bin ) {|t| print ' ', t }
-end
 
-    def printb( i )
-      each_t( @symboltable, i ) do |t|
+    # for debug
+    def print_tab_i(idx, rel, tab, i)
+      bin = tab[i]
+      print i, ' ', idx[i].inspect, ' << '; p rel[i]
+      print '  '
+      each_t(@symboltable, bin) {|t| print ' ', t }
+    end
+
+    # for debug
+    def printb(i)
+      each_t(@symboltable, i) do |t|
         print t, ' '
       end
       puts
     end
 
-    def each_t( tbl, set )
+    def each_t(tbl, set)
       i = ii = idx = nil
       0.upto( set.size ) do |i|
         (0..7).each do |ii|
-          if set[ idx = i * 8 + ii ] == 1 then
+          if set[idx = i * 8 + ii] == 1
             yield tbl[idx]
           end
         end
       end
     end
 
-
     #
     # resolve
     #
 
-    def resolve( state )
-      if state.conflict? then
+    def resolve(state)
+      if state.conflict?
         resolve_rr state, state.ritems
         resolve_sr state, state.stokens
       else
-        if state.rrules.empty? then
+        if state.rrules.empty?
           # shift
           state.stokens.each do |t|
-            state.action[t] = @actions.shift( state.goto_table[t] )
+            state.action[t] = @actions.shift(state.goto_table[t])
           end
         else
           # reduce
-          state.defact = @actions.reduce( state.rrules[0] )
+          state.defact = @actions.reduce(state.rrules[0])
         end
       end
     end
 
-    def resolve_rr( state, r )
+    def resolve_rr(state, r)
       pt = t = act = item = nil
 
       r.each do |item|
-        item.each_la( @symboltable ) do |t|
+        item.each_la(@symboltable) do |t|
           act = state.action[t]
-          if act then
-            Reduce === act or raise "Racc FATAL: #{act.type} in action table"
+          if act
+            raise "racc: fatal: #{act.class} in action table" unless act.kind_of?(Reduce)
             #
             # can't resolve R/R conflict (on t).
             #   reduce with upper rule as default
@@ -431,53 +414,53 @@ end
             state.rr_conflict act.rule, item.rule, t
           else
             # not conflict
-            state.action[t] = @actions.reduce( item.rule )
+            state.action[t] = @actions.reduce(item.rule)
           end
         end
       end
     end
 
-    def resolve_sr( state, s )
+    def resolve_sr(state, s)
       stok = rtok = goto = act = nil
 
       s.each do |stok|
         goto = state.goto_table[stok]
         act = state.action[stok]
 
-        unless act then
+        unless act
           # no conflict
-          state.action[ stok ] = @actions.shift( goto )
+          state.action[stok] = @actions.shift(goto)
         else
-          unless Reduce === act then
+          unless act.kind_of?(Reduce)
             puts 'DEBUG -------------------------------'
             p stok
             p act
             state.action.each do |k,v|
               print k.inspect, ' ', v.inspect, "\n"
             end
-            raise "Racc FATAL: #{act.type} in action table"
+            raise "racc: fatal: #{act.class} in action table"
           end
 
           # conflict on stok
 
           rtok = act.rule.prec
-          case do_resolve_sr( stok, rtok )
+          case do_resolve_sr(stok, rtok)
           when :Reduce
             # action is already set
 
           when :Shift
             # overwrite
             act.decref
-            state.action[ stok ] = @actions.shift( goto )
+            state.action[stok] = @actions.shift(goto)
 
           when :Error
             act.decref
-            state.action[ stok ] = @actions.error
+            state.action[stok] = @actions.error
 
           when :CantResolve
             # shift as default
             act.decref
-            state.action[ stok ] = @actions.shift( goto )
+            state.action[stok] = @actions.shift(goto)
             state.sr_conflict stok, act.rule
           end
         end
@@ -490,65 +473,62 @@ end
       :Nonassoc => :Error
     }
    
-    def do_resolve_sr( stok, rtok )
+    def do_resolve_sr(stok, rtok)
       puts "resolve_sr: s/r conflict: rtok=#{rtok}, stok=#{stok}" if @d_prec
 
-      unless rtok and rtok.prec then
+      unless rtok and rtok.prec
         puts "resolve_sr: no prec for #{rtok}(R)" if @d_prec
         return :CantResolve
       end
       rprec = rtok.prec
 
-      unless stok and stok.prec then
+      unless stok and stok.prec
         puts "resolve_sr: no prec for #{stok}(S)" if @d_prec
         return :CantResolve
       end
       sprec = stok.prec
 
-      ret = if rprec == sprec then
-              ASSOC[ rtok.assoc ] or
-                  raise "Racc FATAL: #{rtok}.assoc is not Left/Right/Nonassoc"
+      ret = if rprec == sprec
+              ASSOC[rtok.assoc] or
+                  raise "racc: fatal: #{rtok}.assoc is not Left/Right/Nonassoc"
             else
-              if rprec > sprec then :Reduce else :Shift end
+              (rprec > sprec) ? (:Reduce) : (:Shift)
             end
 
       puts "resolve_sr: resolved as #{ret.id2name}" if @d_prec
       ret
     end
 
-
     #
     # complete
     #
 
     def set_accept
       anch = @symboltable.anchor
-      init_state = @states[0].goto_table[ @ruletable.start ]
-      targ_state = init_state.action[ anch ].goto_state
-      acc_state  = targ_state.action[ anch ].goto_state
+      init_state = @states[0].goto_table[@ruletable.start]
+      targ_state = init_state.action[anch].goto_state
+      acc_state  = targ_state.action[anch].goto_state
 
       acc_state.action.clear
       acc_state.goto_table.clear
       acc_state.defact = @actions.accept
     end
 
-    def pack( state )
+    def pack(state)
       ### find most frequently used reduce rule
       act = state.action
-      arr = Array.new( @ruletable.size, 0 )
+      arr = Array.new(@ruletable.size, 0)
       t = a = nil
       act.each do |t,a|
-        if Reduce === a then
-          arr[ a.ruleid ] += 1
-        end
+        arr[a.ruleid] += 1 if a.kind_of?(Reduce)
       end
       i = arr.max
       s = i>0 ? arr.index(i) : nil
 
       ### set & delete default action
-      if s then
+      if s
         r = @actions.reduce(s)
-        if not state.defact or state.defact == r then
+        if not state.defact or state.defact == r
           act.delete_if {|t,a| a == r }
           state.defact = r
         end
@@ -561,15 +541,15 @@ end
       act = nil
       used = []
       @actions.each_reduce do |act|
-        if not act or act.refn == 0 then
+        if not act or act.refn == 0
           act.rule.useless = true
         else
           t = act.rule.target
-          used[ t.ident ] = t
+          used[t.ident] = t
         end
       end
-      @symboltable.nt_base.upto( @symboltable.nt_max - 1 ) do |n|
-        unless used[n] then
+      @symboltable.nt_base.upto(@symboltable.nt_max - 1) do |n|
+        unless used[n]
           @symboltable[n].useless = true
         end
       end
@@ -578,55 +558,43 @@ end
   end   # class StateTable
 
 
-  #
-  # State
-  #
-  # stands one lalr state.
-  #
-
+  # A LALR state.
   class State
 
-    def initialize( ident, core )
+    def initialize(ident, core)
       @ident = ident
-      @core  = core
-
+      @core = core
       @goto_table = {}
-      @gotos      = {}
-
+      @gotos = {}
       @stokens = nil
       @ritems = nil
-
       @action = {}
       @defact = nil
-
       @rrconf = nil
       @srconf = nil
 
-      ###
-
-      @closure = make_closure( @core )
+      @closure = make_closure(@core)
     end
 
-
-    attr :ident
+    attr_reader :ident
     alias stateid ident
     alias hash ident
 
-    attr :core
-    attr :closure
+    attr_reader :core
+    attr_reader :closure
 
-    attr :goto_table
-    attr :gotos
+    attr_reader :goto_table
+    attr_reader :gotos
 
-    attr :stokens
-    attr :ritems
-    attr :rrules
+    attr_reader :stokens
+    attr_reader :ritems
+    attr_reader :rrules
 
-    attr :action
-    attr :defact, true   # default action
+    attr_reader :action
+    attr_accessor :defact   # default action
 
-    attr :rrconf
-    attr :srconf
+    attr_reader :rrconf
+    attr_reader :srconf
 
     def inspect
       "<state #{@ident}>"
@@ -634,33 +602,32 @@ end
 
     alias to_s inspect
 
-    def ==( oth )
+    def ==(oth)
       @ident == oth.ident
     end
 
     alias eql? ==
 
-
-    def make_closure( core )
+    def make_closure(core)
       set = ISet.new
       core.each do |ptr|
         set.add ptr
-        if t = ptr.deref and t.nonterminal? then
+        if t = ptr.dereference and t.nonterminal?
           set.update_a t.expand
         end
       end
       set.to_a
     end
 
-    def check_la( la_rules )
+    def check_la(la_rules)
       @conflict = false
       s = []
       r = []
       @closure.each do |ptr|
-        if t = ptr.deref then
-          if t.terminal? then
+        if t = ptr.dereference
+          if t.terminal?
             s[t.ident] = t
-            if t.ident == 1 then   # $error
+            if t.ident == 1    # $error
               @conflict = true
             end
           end
@@ -668,8 +635,8 @@ end
           r.push ptr.rule
         end
       end
-      unless r.empty? then
-        if not s.empty? or r.size > 1 then
+      unless r.empty?
+        if not s.empty? or r.size > 1
           @conflict = true
         end
       end
@@ -677,9 +644,9 @@ end
       @stokens  = s
       @rrules = r
 
-      if @conflict then
+      if @conflict
         @la_rules_i = la_rules.size
-        @la_rules = r.collect {|i| i.ident }
+        @la_rules = r.map {|i| i.ident }
         la_rules.concat r
       else
         @la_rules_i = @la_rules = nil
@@ -690,8 +657,8 @@ end
       @conflict
     end
 
-    def rruleid( rule )
-      if i = @la_rules.index( rule.ident ) then
+    def rruleid(rule)
+      if i = @la_rules.index(rule.ident)
         @la_rules_i + i
       else
         puts '/// rruleid'
@@ -699,37 +666,37 @@ end
         p rule
         p @rrules
         p @la_rules_i
-        raise 'Racc FATAL: cannot get reduce rule id'
+        raise 'racc: fatal: cannot get reduce rule id'
       end
     end
 
-    def set_la( la )
+    def set_la(la)
       return unless @conflict
 
       i = @la_rules_i
       @ritems = r = []
       @rrules.each do |rule|
-        r.push Item.new( rule, la[i] )
+        r.push Item.new(rule, la[i])
         i += 1
       end
     end
 
-    def rr_conflict( high, low, ctok )
-      c = RRconflict.new( @ident, high, low, ctok )
+    def rr_conflict(high, low, ctok)
+      c = RRconflict.new(@ident, high, low, ctok)
 
       @rrconf ||= {}
-      if a = @rrconf[ctok] then
+      if a = @rrconf[ctok]
         a.push c
       else
         @rrconf[ctok] = [c]
       end
     end
 
-    def sr_conflict( shift, reduce )
-      c = SRconflict.new( @ident, shift, reduce )
+    def sr_conflict(shift, reduce)
+      c = SRconflict.new(@ident, shift, reduce)
 
       @srconf ||= {}
-      if a = @srconf[shift] then
+      if a = @srconf[shift]
         a.push c
       else
         @srconf[shift] = [c]
@@ -740,76 +707,59 @@ end
 
 
   #
-  # Goto
-  #
-  # stands one transition on the grammar.
-  # REAL GOTO means transition by nonterminal,
+  # Represents a transition on the grammar.
+  # "Real goto" means a transition by nonterminal,
   # but this class treats also terminal's.
-  # If is terminal transition, .ident returns nil.
+  # If one is a terminal transition, .ident returns nil.
   #
-
   class Goto
-
-    def initialize( ident, sym, from, to )
+    def initialize(ident, sym, from, to)
       @ident      = ident
       @symbol     = sym
       @from_state = from
       @to_state   = to
     end
 
-    attr :ident
-    attr :symbol
-    attr :from_state
-    attr :to_state
+    attr_reader :ident
+    attr_reader :symbol
+    attr_reader :from_state
+    attr_reader :to_state
     
     def inspect
       "(#{@from_state.ident}-#{@symbol}->#{@to_state.ident})"
     end
-  
   end
 
 
-  #
-  # Item
-  #
-  # lalr item. set of rule and its lookahead tokens.
-  #
-
+  # LALR item.  A set of rule and its lookahead tokens.
   class Item
-  
-    def initialize( rule, la )
+    def initialize(rule, la)
       @rule = rule
       @la  = la
     end
 
-    attr :rule
-    attr :la
+    attr_reader :rule
+    attr_reader :la
 
-    def each_la( tbl )
+    def each_la(tbl)
       la = @la
       i = ii = idx = nil
-      0.upto( la.size - 1 ) do |i|
+      0.upto(la.size - 1) do |i|
         (0..7).each do |ii|
-          if la[ idx = i * 8 + ii ] == 1 then
+          if la[idx = i * 8 + ii] == 1
             yield tbl[idx]
           end
         end
       end
     end
-
   end
 
 
-  #
-  # ActionTable
-  #
-  # the table of lalr actions. Actions are
-  # Shift, Reduce, Accept and Error
-  #
-
+  # The table of LALR actions. Actions are either of
+  # Shift, Reduce, Accept and Error.
   class ActionTable
 
-    def initialize( rt, st )
+    def initialize(rt, st)
       @ruletable = rt
       @statetable = st
 
@@ -820,70 +770,68 @@ end
     end
 
     def init
-      @ruletable.each do |rl|
-        @reduce.push Reduce.new( rl )
+      @ruletable.each do |rule|
+        @reduce.push Reduce.new(rule)
       end
-      @statetable.each do |st|
-        @shift.push Shift.new( st )
+      @statetable.each do |state|
+        @shift.push Shift.new(state)
       end
       @accept = Accept.new
       @error = Error.new
     end
 
-
     def reduce_n
       @reduce.size
     end
 
-    def reduce( i )
+    def reduce(i)
       case i
       when Rule    then i = i.ident
       when Integer then ;
-      else              raise "Racc FATAL: wrong type #{i.type} for reduce"
+      else
+        raise "racc: fatal: wrong class #{i.class} for reduce"
       end
 
-      r = @reduce[i] or raise "Racc FATAL: reduce action #{i.inspect} not exist"
+      r = @reduce[i] or raise "racc: fatal: reduce action #{i.inspect} not exist"
       r.incref
       r
     end
 
-    def each_reduce( &block )
-      @reduce.each &block
+    def each_reduce(&block)
+      @reduce.each(&block)
     end
 
-
     def shift_n
       @shift.size
     end
 
-    def shift( i )
+    def shift(i)
       case i
       when State   then i = i.ident
       when Integer then ;
-      else              raise "Racc FATAL: wrong type #{i.type} for shift"
+      else
+        raise "racc: fatal: wrong class #{i.class} for shift"
       end
 
-      @shift[i] or raise "Racc FATAL: shift action #{i} not exist"
+      @shift[i] or raise "racc: fatal: shift action #{i} does not exist"
     end
 
-    def each_shift( &block )
-      @shift.each &block
+    def each_shift(&block)
+      @shift.each(&block)
     end
 
-
-    attr :accept
-    attr :error
+    attr_reader :accept
+    attr_reader :error
 
   end
 
 
   class Shift
-
-    def initialize( goto )
+    def initialize(goto)
       @goto_state = goto
     end
 
-    attr :goto_state
+    attr_reader :goto_state
 
     def goto_id
       @goto_state.ident
@@ -892,19 +840,17 @@ end
     def inspect
       "<shift #{@goto_state.ident}>"
     end
-
   end
 
 
   class Reduce
-
-    def initialize( rule )
+    def initialize(rule)
       @rule = rule
       @refn = 0
     end
 
-    attr :rule
-    attr :refn
+    attr_reader :rule
+    attr_reader :refn
 
     def ruleid
       @rule.ident
@@ -920,74 +866,56 @@ end
 
     def decref
       @refn -= 1
-      if @refn < 0 then
-        bug! 'act.refn < 0'
-      end
+      raise 'racc: fatal: act.refn < 0' if @refn < 0
     end
-
   end
 
-
   class Accept
-
     def inspect
       "<accept>"
     end
-
   end
 
-
   class Error
-
     def inspect
       "<error>"
     end
-  
   end
 
-
-  #
-  # Conflicts
-  #
-
   class SRconflict
-
-    def initialize( sid, shift, reduce )
+    def initialize(sid, shift, reduce)
       @stateid = sid
       @shift   = shift
       @reduce  = reduce
     end
     
-    attr :stateid
-    attr :shift
-    attr :reduce
+    attr_reader :stateid
+    attr_reader :shift
+    attr_reader :reduce
   
     def to_s
-      sprintf( 'state %d: S/R conflict rule %d reduce and shift %s',
-               @stateid, @reduce.ruleid, @shift.to_s )
+      sprintf('state %d: S/R conflict rule %d reduce and shift %s',
+              @stateid, @reduce.ruleid, @shift.to_s)
     end
-
   end
 
   class RRconflict
-    
-    def initialize( sid, high, low, tok )
+    def initialize(sid, high, low, tok)
       @stateid   = sid
       @high_prec = high
       @low_prec  = low
       @token     = tok
     end
 
-    attr :stateid
-    attr :high_prec
-    attr :low_prec
-    attr :token
+    attr_reader :stateid
+    attr_reader :high_prec
+    attr_reader :low_prec
+    attr_reader :token
   
     def to_s
-      sprintf( 'state %d: R/R conflict with rule %d and %d on %s',
-               @stateid, @high_prec.ident, @low_prec.ident, @token.to_s )
+      sprintf('state %d: R/R conflict with rule %d and %d on %s',
+              @stateid, @high_prec.ident, @low_prec.ident, @token.to_s)
     end
-
   end
 
 end
