@@ -1,7 +1,7 @@
 #
-# parser.rb
+# $Id: parser.rb,v 1.7 2005/11/20 17:31:32 aamine Exp $
 #
-# Copyright (c) 1999-2003 Minero Aoki <aamine@loveruby.net>
+# Copyright (c) 1999-2005 Minero Aoki
 #
 # This program is free software.
 # You can distribute/modify this program under the same terms of ruby.
@@ -10,14 +10,11 @@
 # into a Racc output file, you may use that output file
 # without restriction.
 #
-# $Id: parser.rb,v 1.1 2003/10/12 06:10:49 aamine Exp $
-#
 
 unless defined?(NotImplementedError)
   NotImplementedError = NotImplementError
 end
 
-
 module Racc
   class ParseError < StandardError; end
 end
@@ -25,7 +22,6 @@ unless defined?(::ParseError)
   ParseError = Racc::ParseError
 end
 
-
 module Racc
 
   unless defined?(Racc_No_Extentions)
@@ -34,15 +30,15 @@ module Racc
 
   class Parser
 
-    Racc_Runtime_Version = '1.4.3'
-    Racc_Runtime_Revision = '$Revision: 1.1 $'.split(/\s+/)[1]
+    Racc_Runtime_Version = '1.4.5'
+    Racc_Runtime_Revision = '$Revision: 1.7 $'.split[1]
 
-    Racc_Runtime_Core_Version_R = '1.4.3'
-    Racc_Runtime_Core_Revision_R = '$Revision: 1.1 $'.split(/\s+/)[1]
+    Racc_Runtime_Core_Version_R = '1.4.5'
+    Racc_Runtime_Core_Revision_R = '$Revision: 1.7 $'.split[1]
     begin
       require 'racc/cparse'
     # Racc_Runtime_Core_Version_C  = (defined in extention)
-      Racc_Runtime_Core_Revision_C = Racc_Runtime_Core_Id_C.split(/\s+/)[2]
+      Racc_Runtime_Core_Revision_C = Racc_Runtime_Core_Id_C.split[2]
       unless new.respond_to?(:_racc_do_parse_c, true)
         raise LoadError, 'old cparse.so'
       end
@@ -71,9 +67,9 @@ module Racc
 
     def _racc_setup
       @yydebug = false unless self.class::Racc_debug_parser
-      @yydebug = false unless defined? @yydebug
+      @yydebug = false unless defined?(@yydebug)
       if @yydebug
-        @racc_debug_out = $stderr unless defined? @racc_debug_out
+        @racc_debug_out = $stderr unless defined?(@racc_debug_out)
         @racc_debug_out ||= $stderr
       end
       arg = self.class::Racc_arg
@@ -95,20 +91,19 @@ module Racc
       @racc_error_status = 0
     end
 
-
     ###
     ### do_parse
     ###
 
     def do_parse
-      __send__ Racc_Main_Parsing_Routine, _racc_setup(), false
+      __send__(Racc_Main_Parsing_Routine, _racc_setup(), false)
     end
 
     def next_token
       raise NotImplementedError, "#{self.class}\#next_token is not defined"
     end
 
-    def _racc_do_parse_rb( arg, in_debug )
+    def _racc_do_parse_rb(arg, in_debug)
       action_table, action_check, action_default, action_pointer,
       goto_table,   goto_check,   goto_default,   goto_pointer,
       nt_base,      reduce_table, token_table,    shift_n,
@@ -119,34 +114,33 @@ module Racc
       nerr = 0
 
       catch(:racc_end_parse) {
-          while true
-            if i = action_pointer[@racc_state[-1]]
-              if @racc_read_next
-                if @racc_t != 0   # not EOF
-                  tok, @racc_val = next_token()
-                  unless tok      # EOF
-                    @racc_t = 0
-                  else
-                    @racc_t = (token_table[tok] or 1)   # error token
-                  end
-                  racc_read_token(@racc_t, tok, @racc_val) if @yydebug
-                  @racc_read_next = false
+        while true
+          if i = action_pointer[@racc_state[-1]]
+            if @racc_read_next
+              if @racc_t != 0   # not EOF
+                tok, @racc_val = next_token()
+                unless tok      # EOF
+                  @racc_t = 0
+                else
+                  @racc_t = (token_table[tok] or 1)   # error token
                 end
+                racc_read_token(@racc_t, tok, @racc_val) if @yydebug
+                @racc_read_next = false
               end
-              i += @racc_t
-              if  i >= 0 and
-                  act = action_table[i] and
-                  action_check[i] == @racc_state[-1]
-                ;
-              else
-                act = action_default[@racc_state[-1]]
-              end
-            else
+            end
+            i += @racc_t
+            unless i >= 0 and
+                   act = action_table[i] and
+                   action_check[i] == @racc_state[-1]
               act = action_default[@racc_state[-1]]
             end
-            while act = _racc_evalact(act, arg)
-            end
+          else
+            act = action_default[@racc_state[-1]]
           end
+          while act = _racc_evalact(act, arg)
+            ;
+          end
+        end
       }
     end
 
@@ -154,11 +148,11 @@ module Racc
     ### yyparse
     ###
 
-    def yyparse( recv, mid )
-      __send__ Racc_YY_Parse_Method, recv, mid, _racc_setup(), true
+    def yyparse(recv, mid)
+      __send__(Racc_YY_Parse_Method, recv, mid, _racc_setup(), true)
     end
 
-    def _racc_yyparse_rb( recv, mid, arg, c_debug )
+    def _racc_yyparse_rb(recv, mid, arg, c_debug)
       action_table, action_check, action_default, action_pointer,
       goto_table,   goto_check,   goto_default,   goto_pointer,
       nt_base,      reduce_table, token_table,    shift_n,
@@ -170,15 +164,13 @@ module Racc
       i = nil
       nerr = 0
 
-
       catch(:racc_end_parse) {
         until i = action_pointer[@racc_state[-1]]
           while act = _racc_evalact(action_default[@racc_state[-1]], arg)
+            ;
           end
         end
-
         recv.__send__(mid) do |tok, val|
-# $stderr.puts "rd: tok=#{tok}, val=#{val}"
           unless tok
             @racc_t = 0
           else
@@ -188,35 +180,26 @@ module Racc
           @racc_read_next = false
 
           i += @racc_t
-          if  i >= 0 and
-              act = action_table[i] and
-              action_check[i] == @racc_state[-1]
-            ;
-# $stderr.puts "01: act=#{act}"
-          else
+          unless i >= 0 and
+                 act = action_table[i] and
+                 action_check[i] == @racc_state[-1]
             act = action_default[@racc_state[-1]]
-# $stderr.puts "02: act=#{act}"
-# $stderr.puts "curstate=#{@racc_state[-1]}"
           end
-
           while act = _racc_evalact(act, arg)
+            ;
           end
 
           while not (i = action_pointer[@racc_state[-1]]) or
                 not @racc_read_next or
                 @racc_t == 0   # $
-            if i and i += @racc_t and
-               i >= 0 and
-               act = action_table[i] and
-               action_check[i] == @racc_state[-1]
-              ;
-# $stderr.puts "03: act=#{act}"
-            else
-# $stderr.puts "04: act=#{act}"
+            unless i and i += @racc_t and
+                   i >= 0 and
+                   act = action_table[i] and
+                   action_check[i] == @racc_state[-1]
               act = action_default[@racc_state[-1]]
             end
-
             while act = _racc_evalact(act, arg)
+              ;
             end
           end
         end
@@ -227,27 +210,23 @@ module Racc
     ### common
     ###
 
-    def _racc_evalact( act, arg )
-# $stderr.puts "ea: act=#{act}"
+    def _racc_evalact(act, arg)
       action_table, action_check, action_default, action_pointer,
       goto_table,   goto_check,   goto_default,   goto_pointer,
       nt_base,      reduce_table, token_table,    shift_n,
       reduce_n,     use_result,   * = arg
-nerr = 0   # tmp
+      nerr = 0   # tmp
 
       if act > 0 and act < shift_n
         #
         # shift
         #
-
         if @racc_error_status > 0
           @racc_error_status -= 1 unless @racc_t == 1   # error token
         end
-
         @racc_vstack.push @racc_val
         @racc_state.push act
         @racc_read_next = true
-
         if @yydebug
           @racc_tstack.push @racc_t
           racc_shift @racc_t, @racc_tstack, @racc_vstack
@@ -257,10 +236,9 @@ nerr = 0   # tmp
         #
         # reduce
         #
-
         code = catch(:racc_jump) {
-            @racc_state.push _racc_do_reduce(arg, act)
-            false
+          @racc_state.push _racc_do_reduce(arg, act)
+          false
         }
         if code
           case code
@@ -270,7 +248,7 @@ nerr = 0   # tmp
           when 2 # yyaccept
             return shift_n
           else
-            raise RuntimeError, '[Racc Bug] unknown jump code'
+            raise '[Racc Bug] unknown jump code'
           end
         end
 
@@ -278,7 +256,6 @@ nerr = 0   # tmp
         #
         # accept
         #
-
         racc_accept if @yydebug
         throw :racc_end_parse, @racc_vstack[0]
 
@@ -286,7 +263,6 @@ nerr = 0   # tmp
         #
         # error
         #
-
         case @racc_error_status
         when 0
           unless arg[21]    # user_yyerror
@@ -301,7 +277,6 @@ nerr = 0   # tmp
         end
         @racc_user_yyerror = false
         @racc_error_status = 3
-
         while true
           if i = action_pointer[@racc_state[-1]]
             i += 1   # error token
@@ -311,8 +286,7 @@ nerr = 0   # tmp
               break
             end
           end
-
-          throw :racc_end_parse, nil if @racc_state.size < 2
+          throw :racc_end_parse, nil if @racc_state.size <= 1
           @racc_state.pop
           @racc_vstack.pop
           if @yydebug
@@ -320,11 +294,10 @@ nerr = 0   # tmp
             racc_e_pop @racc_state, @racc_tstack, @racc_vstack
           end
         end
-
         return act
 
       else
-        raise RuntimeError, "[Racc Bug] unknown action #{act.inspect}"
+        raise "[Racc Bug] unknown action #{act.inspect}"
       end
 
       racc_next_state(@racc_state[-1], @racc_state) if @yydebug
@@ -332,7 +305,7 @@ nerr = 0   # tmp
       nil
     end
 
-    def _racc_do_reduce( arg, act )
+    def _racc_do_reduce(arg, act)
       action_table, action_check, action_default, action_pointer,
       goto_table,   goto_check,   goto_default,   goto_pointer,
       nt_base,      reduce_table, token_table,    shift_n,
@@ -373,7 +346,7 @@ nerr = 0   # tmp
       goto_default[k1]
     end
 
-    def on_error( t, val, vstack )
+    def on_error(t, val, vstack)
       raise ParseError, sprintf("\nparse error on value %s (%s)",
                                 val.inspect, token_to_str(t) || '?')
     end
@@ -390,23 +363,24 @@ nerr = 0   # tmp
       @racc_error_status = 0
     end
 
-
+    #
     # for debugging output
+    #
 
-    def racc_read_token( t, tok, val )
+    def racc_read_token(t, tok, val)
       @racc_debug_out.print 'read    '
       @racc_debug_out.print tok.inspect, '(', racc_token2str(t), ') '
       @racc_debug_out.puts val.inspect
       @racc_debug_out.puts
     end
 
-    def racc_shift( tok, tstack, vstack )
+    def racc_shift(tok, tstack, vstack)
       @racc_debug_out.puts "shift   #{racc_token2str tok}"
       racc_print_stacks tstack, vstack
       @racc_debug_out.puts
     end
 
-    def racc_reduce( toks, sim, tstack, vstack )
+    def racc_reduce(toks, sim, tstack, vstack)
       out = @racc_debug_out
       out.print 'reduce '
       if toks.empty?
@@ -425,20 +399,20 @@ nerr = 0   # tmp
       @racc_debug_out.puts
     end
 
-    def racc_e_pop( state, tstack, vstack )
+    def racc_e_pop(state, tstack, vstack)
       @racc_debug_out.puts 'error recovering mode: pop token'
       racc_print_states state
       racc_print_stacks tstack, vstack
       @racc_debug_out.puts
     end
 
-    def racc_next_state( curstate, state )
+    def racc_next_state(curstate, state)
       @racc_debug_out.puts  "goto    #{curstate}"
       racc_print_states state
       @racc_debug_out.puts
     end
 
-    def racc_print_stacks( t, v )
+    def racc_print_stacks(t, v)
       out = @racc_debug_out
       out.print '        ['
       t.each_index do |i|
@@ -447,19 +421,19 @@ nerr = 0   # tmp
       out.puts ' ]'
     end
 
-    def racc_print_states( s )
+    def racc_print_states(s)
       out = @racc_debug_out
       out.print '        ['
       s.each {|st| out.print ' ', st }
       out.puts ' ]'
     end
 
-    def racc_token2str( tok )
+    def racc_token2str(tok)
       self.class::Racc_token_to_s_table[tok] or
-          raise RuntimeError, "[Racc Bug] can't convert token #{tok} to string"
+          raise "[Racc Bug] can't convert token #{tok} to string"
     end
 
-    def token_to_str( t )
+    def token_to_str(t)
       self.class::Racc_token_to_s_table[t]
     end
 
